@@ -49,61 +49,140 @@ struct Init
 
 struct gstreamer::Engine::Private
 {
-    void on_playbin_state_changed(
-            const gstreamer::Bus::Message::Detail::StateChanged& state_change)
+    media::Player::PlaybackStatus gst_state_to_player_status(const gstreamer::Bus::Message::Detail::StateChanged& state)
     {
-        (void) state_change;
+        if (state.new_state == GST_STATE_PLAYING)
+            return media::Player::PlaybackStatus::playing;
+        else if (state.new_state == GST_STATE_PAUSED)
+            return media::Player::PlaybackStatus::paused;
+        else if (state.new_state == GST_STATE_READY)
+            return media::Player::PlaybackStatus::ready;
+        else if (state.new_state == GST_STATE_NULL)
+            return media::Player::PlaybackStatus::null;
+        else
+            return media::Player::PlaybackStatus::stopped;
+    }
+
+    void on_playbin_state_changed(const std::pair<gstreamer::Bus::Message::Detail::StateChanged,std::string>& p)
+    {
+        if (p.second == "playbin")
+        {
+            std::cout << "State changed on playbin: "
+                      << gst_element_state_get_name(p.first.new_state) << std::endl;
+            const auto status = gst_state_to_player_status(p.first);
+            /*
+             * When state moves to "paused" the pipeline is already set. We check that we
+             * have streams to play.
+             */
+            if (status == media::Player::PlaybackStatus::paused &&
+                    !playbin.can_play_streams()) {
+                std::cerr << "** Cannot play: some codecs are missing" << std::endl;
+                playbin.reset();
+                const media::Player::Error e = media::Player::Error::format_error;
+                error(e);
+            } else {
+                playback_status_changed(status);
+            }
+        }
     }
 
     // Converts from a GStreamer GError to a media::Player:Error enum
     media::Player::Error from_gst_errorwarning(const gstreamer::Bus::Message::Detail::ErrorWarningInfo& ewi)
     {
+        media::Player::Error ret_error = media::Player::Error::no_error;
+
         if (g_strcmp0(g_quark_to_string(ewi.error->domain), "gst-core-error-quark") == 0)
         {
             switch (ewi.error->code)
             {
+            case GST_CORE_ERROR_FAILED:
+                std::cerr << "** Encountered a GST_CORE_ERROR_FAILED" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_CORE_ERROR_NEGOTIATION:
-                return media::Player::Error::resource_error;
+                std::cerr << "** Encountered a GST_CORE_ERROR_NEGOTIATION" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_CORE_ERROR_MISSING_PLUGIN:
-                return media::Player::Error::format_error;
+                std::cerr << "** Encountered a GST_CORE_ERROR_MISSING_PLUGIN" << std::endl;
+                ret_error = media::Player::Error::format_error;
+                break;
             default:
-                std::cerr << "Got an unhandled core error: '"
+                std::cerr << "** Encountered an unhandled core error: '"
                     << ewi.debug << "' (code: " << ewi.error->code << ")" << std::endl;
-                return media::Player::Error::no_error;
+                ret_error = media::Player::Error::no_error;
+                break;
             }
         }
         else if (g_strcmp0(g_quark_to_string(ewi.error->domain), "gst-resource-error-quark") == 0)
         {
             switch (ewi.error->code)
             {
+            case GST_RESOURCE_ERROR_FAILED:
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_FAILED" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_NOT_FOUND:
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_NOT_FOUND" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_OPEN_READ:
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_OPEN_READ" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_OPEN_WRITE:
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_OPEN_WRITE" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_READ:
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_READ" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_WRITE:
-                return media::Player::Error::resource_error;
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_WRITE" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_RESOURCE_ERROR_NOT_AUTHORIZED:
-                return media::Player::Error::access_denied_error;
+                std::cerr << "** Encountered a GST_RESOURCE_ERROR_NOT_AUTHORIZED" << std::endl;
+                ret_error = media::Player::Error::access_denied_error;
+                break;
             default:
-                std::cerr << "Got an unhandled resource error: '"
+                std::cerr << "** Encountered an unhandled resource error: '"
                     << ewi.debug << "' (code: " << ewi.error->code << ")" << std::endl;
-                return media::Player::Error::no_error;
+                ret_error = media::Player::Error::no_error;
+                break;
             }
         }
         else if (g_strcmp0(g_quark_to_string(ewi.error->domain), "gst-stream-error-quark") == 0)
         {
             switch (ewi.error->code)
             {
+            case GST_STREAM_ERROR_FAILED:
+                std::cerr << "** Encountered a GST_STREAM_ERROR_FAILED" << std::endl;
+                ret_error = media::Player::Error::resource_error;
+                break;
             case GST_STREAM_ERROR_CODEC_NOT_FOUND:
-                return media::Player::Error::format_error;
+                std::cerr << "** Encountered a GST_STREAM_ERROR_CODEC_NOT_FOUND" << std::endl;
+                // Missing codecs are handled later, when state switches to "paused"
+                ret_error = media::Player::Error::no_error;
+                break;
+            case GST_STREAM_ERROR_DECODE:
+                std::cerr << "** Encountered a GST_STREAM_ERROR_DECODE" << std::endl;
+                ret_error = media::Player::Error::format_error;
+                break;
             default:
-                std::cerr << "Got an unhandled stream error: '"
+                std::cerr << "** Encountered an unhandled stream error: '"
                     << ewi.debug << "' (code: " << ewi.error->code << ")" << std::endl;
-                return media::Player::Error::no_error;
+                ret_error = media::Player::Error::no_error;
+                break;
             }
         }
 
-        return media::Player::Error::no_error;
+        if (ret_error != media::Player::Error::no_error) {
+            std::cerr << "Resetting playbin pipeline after unrecoverable error" << std::endl;
+            playbin.reset();
+        }
+        return ret_error;
     }
 
     void on_playbin_error(const gstreamer::Bus::Message::Detail::ErrorWarningInfo& ewi)
@@ -312,13 +391,13 @@ struct gstreamer::Engine::Private
 
 gstreamer::Engine::Engine() : d(new Private{})
 {
-    cout << "Creating a new Engine instance in " << __PRETTY_FUNCTION__ << endl;
-    d->state = media::Engine::State::ready;
+    d->state = media::Engine::State::no_media;
 }
 
 gstreamer::Engine::~Engine()
 {
     stop();
+    d->state = media::Engine::State::no_media;
 }
 
 const std::shared_ptr<media::Engine::MetaDataExtractor>& gstreamer::Engine::meta_data_extractor() const
@@ -331,9 +410,9 @@ const core::Property<media::Engine::State>& gstreamer::Engine::state() const
     return d->state;
 }
 
-bool gstreamer::Engine::open_resource_for_uri(const media::Track::UriType& uri)
+bool gstreamer::Engine::open_resource_for_uri(const media::Track::UriType& uri, bool do_pipeline_reset)
 {
-    d->playbin.set_uri(uri, core::ubuntu::media::Player::HeadersType{});
+    d->playbin.set_uri(uri, core::ubuntu::media::Player::HeadersType{}, do_pipeline_reset);
     return true;
 }
 
@@ -350,12 +429,13 @@ void gstreamer::Engine::create_video_sink(uint32_t texture_id)
 
 bool gstreamer::Engine::play()
 {
-    auto result = d->playbin.set_state_and_wait(GST_STATE_PLAYING);
+    const auto result = d->playbin.set_state_and_wait(GST_STATE_PLAYING);
 
     if (result)
     {
         d->state = media::Engine::State::playing;
         cout << __PRETTY_FUNCTION__ << endl;
+        cout << "Engine: playing uri: " << d->playbin.uri() << endl;
         d->playback_status_changed(media::Player::PlaybackStatus::playing);
     }
 
@@ -366,10 +446,12 @@ bool gstreamer::Engine::stop()
 {
     // No need to wait, and we can immediately return.
     if (d->state == media::Engine::State::stopped)
+    {
+        std::cerr << "Current player state is already stopped - no need to change state to stopped" << std::endl;
         return true;
+    }
 
-    auto result = d->playbin.set_state_and_wait(GST_STATE_NULL);
-
+    const auto result = d->playbin.set_state_and_wait(GST_STATE_NULL);
     if (result)
     {
         d->state = media::Engine::State::stopped;
@@ -382,7 +464,7 @@ bool gstreamer::Engine::stop()
 
 bool gstreamer::Engine::pause()
 {
-    auto result = d->playbin.set_state_and_wait(GST_STATE_PAUSED);
+    const auto result = d->playbin.set_state_and_wait(GST_STATE_PAUSED);
 
     if (result)
     {
