@@ -177,7 +177,7 @@ public:
             bus.on_new_message.connect(
                     [&](const gstreamer::Bus::Message& msg)
                     {
-                        std::cout << __PRETTY_FUNCTION__ << gst_message_type_get_name(msg.type) << std::endl;
+                        //std::cout << __PRETTY_FUNCTION__ << gst_message_type_get_name(msg.type) << std::endl;
                         if (msg.type == GST_MESSAGE_TAG)
                         {
                             MetaDataExtractor::on_tag_available(msg.detail.tag, meta_data);
@@ -191,7 +191,7 @@ public:
         g_object_set(decoder, "uri", uri.c_str(), NULL);
         gst_element_set_state(pipe, GST_STATE_PAUSED);
 
-        if (std::future_status::ready != future.wait_for(std::chrono::seconds(2)))
+        if (std::future_status::ready != future.wait_for(std::chrono::seconds(4)))
         {
             gst_element_set_state(pipe, GST_STATE_NULL);
             throw std::runtime_error("Problem extracting meta data for track");
