@@ -1,4 +1,5 @@
 /*
+ * Copyright © 2013-2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -23,7 +24,6 @@
 
 #include "client_death_observer.h"
 #include "engine.h"
-#include "null_track_list.h"
 #include "track_list_implementation.h"
 
 #include "gstreamer/engine.h"
@@ -58,14 +58,20 @@ struct media::PlayerImplementation<Parent>::Private :
           display_state_lock(config.power_state_controller->display_state_lock()),
           system_state_lock(config.power_state_controller->system_state_lock()),
           engine(std::make_shared<gstreamer::Engine>()),
-          track_list(std::make_shared<NullTrackList>()),
+          track_list(std::make_shared<TrackListImplementation>(
+              config.parent.bus,
+              config.parent.service->add_object_for_path(
+                  dbus::types::ObjectPath(config.parent.session->path().as_string() + "/TrackList")),
+              engine->meta_data_extractor(),
+              config.parent.request_context_resolver,
+              config.parent.request_authenticator)),
           system_wakelock_count(0),
           display_wakelock_count(0),
           previous_state(Engine::State::stopped),
           engine_state_change_connection(engine->state().changed().connect(make_state_change_handler())),
-          engine_playback_status_change_connection(engine->playback_status_changed_signal().connect(make_playback_status_change_handler()))
+          engine_playback_status_change_connection(engine->playback_status_changed_signal().connect(make_playback_status_change_handler())),
+          doing_abandon(false)
     {
-        std::cout << "Private parent instance: " << parent << std::endl;
         // Poor man's logging of release/acquire events.
         display_state_lock->acquired().connect([](media::power::DisplayState state)
         {
@@ -99,7 +105,6 @@ struct media::PlayerImplementation<Parent>::Private :
         // by disconnecting the state change signal
         engine_state_change_connection.disconnect();
 
-        std::cout << "** Disconnecting playback_status_changed_signal connection";
         // The engine destructor can lead to a playback status change which will
         // trigger the playback status change handler. Ensure the handler is not called
         // by disconnecting the playback status change signal
@@ -169,7 +174,7 @@ struct media::PlayerImplementation<Parent>::Private :
     {
         return [this](const media::Player::PlaybackStatus& status)
         {
-            std::cout << "Emiting playback_status_changed for parent: " << parent << std::endl;
+            std::cout << "Emiting playback_status_changed signal: " << status << std::endl;
             parent->emit_playback_status_changed(status);
         };
     }
@@ -282,6 +287,20 @@ struct media::PlayerImplementation<Parent>::Private :
         engine->reset();
     }
 
+    void open_first_track_from_tracklist(const media::Track::Id& id)
+    {
+        const Track::UriType uri = track_list->query_uri_for_track(id);
+        if (!uri.empty())
+        {
+            // Using a TrackList for playback, added tracks via add_track(), but open_uri hasn't been called yet
+            // to load a media resource
+            std::cout << "Calling d->engine->open_resource_for_uri() for first track added only: " << uri << std::endl;
+            std::cout << "\twith a Track::Id: " << id << std::endl;
+            static const bool do_pipeline_reset = false;
+            engine->open_resource_for_uri(uri, do_pipeline_reset);
+        }
+    }
+
     // Our link back to our parent.
     media::PlayerImplementation<Parent>* parent;
     // We just store the parameters passed on construction.
@@ -290,13 +309,16 @@ struct media::PlayerImplementation<Parent>::Private :
     media::power::StateController::Lock<media::power::SystemState>::Ptr system_state_lock;
 
     std::shared_ptr<Engine> engine;
-    std::shared_ptr<media::NullTrackList> track_list;
+    std::shared_ptr<media::TrackListImplementation> track_list;
     std::atomic<int> system_wakelock_count;
     std::atomic<int> display_wakelock_count;
     Engine::State previous_state;
     core::Signal<> on_client_disconnected;
     core::Connection engine_state_change_connection;
     core::Connection engine_playback_status_change_connection;
+    // Prevent the TrackList from auto advancing to the next track
+    std::mutex doing_go_to_track;
+    std::atomic<bool> doing_abandon;
 };
 
 template<typename Parent>
@@ -308,11 +330,11 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
     Parent::can_play().set(true);
     Parent::can_pause().set(true);
     Parent::can_seek().set(true);
-    Parent::can_go_previous().set(true);
-    Parent::can_go_next().set(true);
+    Parent::can_go_previous().set(false);
+    Parent::can_go_next().set(false);
     Parent::is_video_source().set(false);
     Parent::is_audio_source().set(false);
-    Parent::is_shuffle().set(true);
+    Parent::shuffle().set(false);
     Parent::playback_rate().set(1.f);
     Parent::playback_status().set(Player::PlaybackStatus::null);
     Parent::loop_status().set(Player::LoopStatus::none);
@@ -352,6 +374,31 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
     };
     Parent::is_audio_source().install(audio_type_getter);
 
+    std::function<bool()> can_go_next_getter = [this]()
+    {
+        return d->track_list->has_next();
+    };
+    Parent::can_go_next().install(can_go_next_getter);
+
+    std::function<bool()> can_go_previous_getter = [this]()
+    {
+        return d->track_list->has_previous();
+    };
+    Parent::can_go_previous().install(can_go_previous_getter);
+
+    // When the client changes the loop status, make sure to update the TrackList
+    Parent::loop_status().changed().connect([this](media::Player::LoopStatus loop_status)
+    {
+        std::cout << "LoopStatus: " << loop_status << std::endl;
+        d->track_list->on_loop_status_changed(loop_status);
+    });
+
+    // When the client changes the shuffle setting, make sure to update the TrackList
+    Parent::shuffle().changed().connect([this](bool shuffle)
+    {
+        d->track_list->on_shuffle_changed(shuffle);
+    });
+
     // Make sure that the audio_stream_role property gets updated on the Engine side
     // whenever the client side sets the role
     Parent::audio_stream_role().changed().connect([this](media::Player::AudioStreamRole new_role)
@@ -373,14 +420,28 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
 
     d->engine->about_to_finish_signal().connect([this]()
     {
+        if (d->doing_abandon)
+            return;
+
+        // Prevent on_go_to_track from executing as it's not needed in this case. on_go_to_track
+        // (see the lambda below) is only needed when the client explicitly calls next() not during
+        // the about_to_finish condition
+        d->doing_go_to_track.lock();
+
         Parent::about_to_finish()();
 
-        if (d->track_list->has_next())
+        const media::Track::Id prev_track_id = d->track_list->current();
+        // Make sure that the TrackList keeps advancing. The logic for what gets played next,
+        // if anything at all, occurs in TrackListSkeleton::next()
+        const Track::UriType uri = d->track_list->query_uri_for_track(d->track_list->next());
+        if (prev_track_id != d->track_list->current() && !uri.empty())
         {
-            Track::UriType uri = d->track_list->query_uri_for_track(d->track_list->next());
-            if (!uri.empty())
-                d->parent->open_uri(uri);
+            std::cout << "Advancing to next track on playbin: " << uri << std::endl;
+            static const bool do_pipeline_reset = false;
+            d->engine->open_resource_for_uri(uri, do_pipeline_reset);
         }
+
+        d->doing_go_to_track.unlock();
     });
 
     d->engine->client_disconnected_signal().connect([this]()
@@ -388,6 +449,7 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
         // If the client disconnects, make sure both wakelock types
         // are cleared
         d->clear_wakelocks();
+        d->track_list->reset();
         // And tell the outside world that the client has gone away
         d->on_client_disconnected();
     });
@@ -412,6 +474,54 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
         Parent::error()(e);
     });
 
+    d->track_list->on_end_of_tracklist().connect([this]()
+    {
+        if (d->engine->state() != gstreamer::Engine::State::ready
+                && d->engine->state() != gstreamer::Engine::State::stopped)
+        {
+            std::cout << "End of tracklist reached, stopping playback" << std::endl;
+            d->engine->stop();
+        }
+    });
+
+
+    d->track_list->on_go_to_track().connect([this](std::pair<const media::Track::Id, bool> p)
+    {
+        // This lambda needs to be mutually exclusive with the about_to_finish lambda above
+        const bool locked = d->doing_go_to_track.try_lock();
+        // If the try_lock fails, it means that about_to_finish lambda above has it locked and it will
+        // call d->engine->open_resource_for_uri()
+        if (!locked)
+            return;
+
+        const media::Track::Id id = p.first;
+        const bool toggle_player_state = p.second;
+
+        if (toggle_player_state)
+            d->engine->stop();
+
+        const Track::UriType uri = d->track_list->query_uri_for_track(id);
+        if (!uri.empty())
+        {
+            std::cout << "Setting next track on playbin (on_go_to_track signal): " << uri << std::endl;
+            std::cout << "\twith a Track::Id: " << id << std::endl;
+            static const bool do_pipeline_reset = true;
+            d->engine->open_resource_for_uri(uri, do_pipeline_reset);
+        }
+
+        if (toggle_player_state)
+            d->engine->play();
+
+        d->doing_go_to_track.unlock();
+    });
+
+    d->track_list->on_track_added().connect([this](const media::Track::Id& id)
+    {
+        std::cout << "** Track was added, handling in PlayerImplementation" << std::endl;
+        if (d->track_list->tracks()->size() == 1)
+            d->open_first_track_from_tracklist(id);
+    });
+
     // Everything is setup, we now subscribe to death notifications.
     std::weak_ptr<Private> wp{d};
 
@@ -420,6 +530,9 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
     {
         if (auto sp = wp.lock())
         {
+            if (sp->doing_abandon)
+                return;
+
             if (died != sp->config.key)
                 return;
 
@@ -464,6 +577,27 @@ media::PlayerImplementation<Parent>::~PlayerImplementation()
     Parent::is_audio_source().install(audio_type_getter);
 }
 
+template<typename Parent>
+std::string media::PlayerImplementation<Parent>::uuid() const
+{
+    // No impl for now, as not needed internally.
+    return std::string{};
+}
+
+template<typename Parent>
+void media::PlayerImplementation<Parent>::reconnect()
+{
+    d->config.client_death_observer->register_for_death_notifications_with_key(d->config.key);
+}
+
+template<typename Parent>
+void media::PlayerImplementation<Parent>::abandon()
+{
+    // Signal client disconnection due to abandonment of player
+    d->doing_abandon = true;
+    d->on_client_died();
+}
+
 template<typename Parent>
 std::shared_ptr<media::TrackList> media::PlayerImplementation<Parent>::track_list()
 {
@@ -487,7 +621,12 @@ media::video::Sink::Ptr media::PlayerImplementation<Parent>::create_gl_texture_v
 template<typename Parent>
 bool media::PlayerImplementation<Parent>::open_uri(const Track::UriType& uri)
 {
-    return d->engine->open_resource_for_uri(uri);
+    d->track_list->reset();
+    const bool ret = d->engine->open_resource_for_uri(uri, false);
+    // Don't set new track as the current track to play since we're calling open_resource_for_uri above
+    static const bool make_current = false;
+    d->track_list->add_track_with_uri_at(uri, media::TrackList::after_empty_track(), make_current);
+    return ret;
 }
 
 template<typename Parent>
@@ -499,11 +638,13 @@ bool media::PlayerImplementation<Parent>::open_uri(const Track::UriType& uri, co
 template<typename Parent>
 void media::PlayerImplementation<Parent>::next()
 {
+    d->track_list->next();
 }
 
 template<typename Parent>
 void media::PlayerImplementation<Parent>::previous()
 {
+    d->track_list->previous();
 }
 
 template<typename Parent>
