@@ -43,10 +43,14 @@ namespace media = core::ubuntu::media;
 struct media::PlayerStub::Private
 {
     Private(const std::shared_ptr<Service>& parent,
-            const std::shared_ptr<core::dbus::Object>& object
+            const std::shared_ptr<core::dbus::Service>& service,
+            const std::shared_ptr<core::dbus::Object>& object,
+            const std::string& uuid
             ) : parent(parent),
+                service(service),
                 object(object),
                 key(object->invoke_method_synchronously<mpris::Player::Key, media::Player::PlayerKey>().value()),
+                uuid(uuid),
                 sink_factory(media::video::make_platform_default_sink_factory(key)),
                 properties
                 {
@@ -91,8 +95,10 @@ struct media::PlayerStub::Private
 
     std::shared_ptr<Service> parent;
     std::shared_ptr<TrackList> track_list;
+    dbus::Service::Ptr service;
     dbus::Object::Ptr object;
     media::Player::PlayerKey key;
+    std::string uuid;
     media::video::SinkFactory sink_factory;
     struct
     {
@@ -108,7 +114,7 @@ struct media::PlayerStub::Private
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::TypedPlaybackStatus>> playback_status;
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::TypedLoopStatus>> loop_status;
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::PlaybackRate>> playback_rate;
-        std::shared_ptr<core::dbus::Property<mpris::Player::Properties::Shuffle>> is_shuffle;
+        std::shared_ptr<core::dbus::Property<mpris::Player::Properties::Shuffle>> shuffle;
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::TypedMetaData>> meta_data_for_current_track;
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::Volume>> volume;
         std::shared_ptr<core::dbus::Property<mpris::Player::Properties::Position>> position;
@@ -153,7 +159,7 @@ struct media::PlayerStub::Private
         {
             dbus.seeked_to->connect([this](std::uint64_t value)
             {
-                std::cout << "seeked_to signal arrived via the bus." << std::endl;
+                std::cout << "SeekedTo signal arrived via the bus." << std::endl;
                 seeked_to(value);
             });
 
@@ -171,7 +177,7 @@ struct media::PlayerStub::Private
 
             dbus.playback_status_changed->connect([this](const media::Player::PlaybackStatus& status)
             {
-                std::cout << "PlaybackStatusChanged signal arrived via the bus." << std::endl;
+                std::cout << "PlaybackStatusChanged signal arrived via the bus (Status: " << status << ")" << std::endl;
                 playback_status_changed(status);
             });
 
@@ -209,8 +215,10 @@ struct media::PlayerStub::Private
 
 media::PlayerStub::PlayerStub(
     const std::shared_ptr<Service>& parent,
-    const std::shared_ptr<core::dbus::Object>& object)
-        : d(new Private{parent, object})
+    const std::shared_ptr<core::dbus::Service>& service,
+    const std::shared_ptr<core::dbus::Object>& object,
+    const std::string& uuid)
+        : d(new Private{parent, service, object, uuid})
 {
 }
 
@@ -218,13 +226,30 @@ media::PlayerStub::~PlayerStub()
 {
 }
 
+std::string media::PlayerStub::uuid() const
+{
+    return d->uuid;
+}
+
+void media::PlayerStub::reconnect()
+{
+    // No implementation
+}
+
+void media::PlayerStub::abandon()
+{
+    // No implementation
+}
+
 std::shared_ptr<media::TrackList> media::PlayerStub::track_list()
 {
     if (!d->track_list)
     {
         d->track_list = std::make_shared<media::TrackListStub>(
                     shared_from_this(),
-                    dbus::types::ObjectPath(d->object->path().as_string() + "/TrackList"));
+                    d->service->object_for_path(
+                        dbus::types::ObjectPath(
+                            d->object->path().as_string() + "/TrackList")));
     }
     return d->track_list;
 }
@@ -362,9 +387,9 @@ const core::Property<media::Player::PlaybackRate>& media::PlayerStub::playback_r
     return *d->properties.playback_rate;
 }
 
-const core::Property<bool>& media::PlayerStub::is_shuffle() const
+const core::Property<bool>& media::PlayerStub::shuffle() const
 {
-    return *d->properties.is_shuffle;
+    return *d->properties.shuffle;
 }
 
 const core::Property<media::Track::MetaData>& media::PlayerStub::meta_data_for_current_track() const
@@ -422,9 +447,9 @@ core::Property<media::Player::PlaybackRate>& media::PlayerStub::playback_rate()
     return *d->properties.playback_rate;
 }
 
-core::Property<bool>& media::PlayerStub::is_shuffle()
+core::Property<bool>& media::PlayerStub::shuffle()
 {
-    return *d->properties.is_shuffle;
+    return *d->properties.shuffle;
 }
 
 core::Property<media::Player::Volume>& media::PlayerStub::volume()
