@@ -40,6 +40,11 @@ class ServiceStub : public core::dbus::Stub<core::ubuntu::media::Service>
     ~ServiceStub();
 
     std::shared_ptr<Player> create_session(const Player::Configuration&);
+    void detach_session(const std::string& uuid, const Player::Configuration&);
+    std::shared_ptr<Player> reattach_session(const std::string& uuid, const Player::Configuration&);
+    void destroy_session(const std::string& uuid, const Player::Configuration&);
+    std::shared_ptr<Player> create_fixed_session(const std::string& name, const Player::Configuration&);
+    std::shared_ptr<Player> resume_session(Player::PlayerKey key);
     void pause_other_sessions(Player::PlayerKey key);
 
   private:
